@@ -1,102 +1,4 @@
 
-#####################################################
-### Adding positive SCOP-spline construction 
-######################################################
-
-
-smooth.construct.po.smooth.spec<- function(object, data, knots)
-## construction of the positivelt costrained smooth
-{ 
-   #require(splines)
-  m <- object$p.order[1]
-  if (is.na(m)) m <- 2 ## default for cubic spline
-  if (m<1) stop("silly m supplied")
-  if (object$bs.dim<0) object$bs.dim <- 10 ## default
-  q <- object$bs.dim 
-  nk <- q+m+2 ## number of knots
-  if (nk<=0) stop("k too small for m")
-  x <- data[[object$term]]  ## the data
-  xk <- knots[[object$term]] ## will be NULL if none supplied
-  if (is.null(xk)) # space knots through data
-     { n<-length(x)
-       xk<-rep(0,q+m+2)
-       xk[(m+2):(q+1)]<-seq(min(x),max(x),length=q-m)
-       for (i in 1:(m+1)) {xk[i]<-xk[m+2]-(m+2-i)*(xk[m+3]-xk[m+2])}
-       for (i in (q+2):(q+m+2)) {xk[i]<-xk[q+1]+(i-q-1)*(xk[m+3]-xk[m+2])}
-     }
-  if (length(xk)!=nk) # right number of knots?
-  stop(paste("there should be ", nk," supplied knots"))
-  #  get model matrix-------------
-  X1 <- splineDesign(xk,x,ord=m+2)
-  # use matrix Sigma and remove the first column for the positive smooth
-  Sig <- diag(1,(q-1))  ## identity matrix 
-     
-  X <- X1[,2:q] ## %*%Sig # model submatrix for the positive term
-  object$X <- X # the finished model matrix
-  object$P <- list()
-  object$S <- list()
-  object$Sigma <- Sig
-
-  if (!object$fixed) # create the penalty matrix
-  { P <- diff(diag(q-1),difference=1)
-    object$P[[1]] <- P
-    object$S[[1]] <- crossprod(P)
-  }
-  b <- rep(1,q-1) # define vector of 0's & 1's for model parameters identification
-  object$p.ident <- b  
-  object$rank <- ncol(object$X)-1  # penalty rank
-  object$null.space.dim <- m+1  # dim. of unpenalized space
-  object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
-  object$knots <- xk;
-  object$m <- m;
-  object$df<-ncol(object$X)     # maximum DoF (if unconstrained)
-
-  ## get model matrix for 1st and 2nd derivatives of the smooth...
-  h <- (max(x)-min(x))/(q-m-1) ## distance between two adjacent knots
-  object$Xdf1 <- splineDesign(xk,x,ord=m+1)[,2:(q-1)]/h ## ord is by one less for the 1st derivative
-  object$Xdf2 <- splineDesign(xk,x,ord=m)[,2:(q-2)]/h^2 ## ord is by two less for the 2nd derivative
-
-  class(object)<-"po.smooth"  # Give object a class
-  object
-}
-
-## Prediction matrix for the `po` smooth class... 
-
-Predict.matrix.po.smooth<-function(object,data)
-## prediction method function for the `po' smooth class
-{ m <- object$m+1; # spline order, m+1=3 default for cubic spline
-  q <- object$df +1
-  Sig <- diag(1,q)   # Define Matrix Sigma
-  
-  ## find spline basis inner knot range...
-  ll <- object$knots[m+1];ul <- object$knots[length(object$knots)-m]
-  m <- m + 1
-  x <- data[[object$term]]
-  n <- length(x)
-  ind <- x<=ul & x>=ll ## data in range
-  if (sum(ind)==n) { ## all in range
-     X <- spline.des(object$knots,x,m)$design
-    # X <- X%*%Sig 
-  } else { ## some extrapolation needed 
-     ## matrix mapping coefs to value and slope at end points...
-     D <- spline.des(object$knots,c(ll,ll,ul,ul),m,c(0,1,0,1))$design
-     X <- matrix(0,n,ncol(D)) ## full predict matrix
-     if (sum(ind)> 0)  X[ind,] <- spline.des(object$knots,x[ind],m)$design ## interior rows
-     ## Now add rows for linear extrapolation...
-     ind <- x < ll 
-     if (sum(ind)>0) X[ind,] <- cbind(1,x[ind]-ll)%*%D[1:2,]
-     ind <- x > ul
-     if (sum(ind)>0) X[ind,] <- cbind(1,x[ind]-ul)%*%D[3:4,]
-    # X <- X%*%Sig 
-  }
-  X
-}
-
-
-
-
-
-
 
 #####################################################
 ### Adding Monotone increasing SCOP-spline construction 
@@ -125,25 +27,51 @@ smooth.construct.mpi.smooth.spec<- function(object, data, knots)
      }
   if (length(xk)!=nk) # right number of knots?
   stop(paste("there should be ", nk," supplied knots"))
+  ##################################################
+  ## setting column centering constraint... 
+  #  get model matrix-------------
+#  X1 <- splineDesign(xk,x,ord=m+2)
+  # get matrix Sigma for monotone smooth...
+#  Sig <- matrix(0,q,q)   
+     # elements of matrix Sigma for increasing smooth
+#  for (i in 1:q)  Sig[i,1:i]<-1
+#  X <- (X1%*%Sig)[,-q] # model matrix for the monotone term
+#  cmx <- colMeans(X)
+#  X <- sweep(X,2,cmx) ## subtract cmx from columns 
+#  object$X <- X # the finished model matrix
+#  object$P <- list()
+#  object$S <- list()
+#  object$Sigma <- Sig
+
+#  if (!object$fixed) # create the penalty matrix
+#  { P <- diff(diag(q-1),difference=1)
+#    P[1,] <-0
+#    P[,1] <- 0
+#    object$P[[1]] <- P
+#    object$S[[1]] <- crossprod(P)
+#  }
+#  b <- c(0,rep(1,q-2)) # define vector of 0's & 1's for model parameters identification
+                  # with `1' - for a parameter to be exponentiated, `0' - otherwise
+#######################################################################
+## indentifiability constraint by dropping the first columns of X and Sigma and setting beta_1=0
   #  get model matrix-------------
   X1 <- splineDesign(xk,x,ord=m+2)
-  # use matrix Sigma and remove the first column for the monotone smooth
-  Sig <- matrix(0,(q-1),(q-1))   
-     # elements of matrix Sigma for increasing smooth
-  for (i in 1:(q-1))  Sig[i,1:i]<-1
-  X <- X1[,2:q]%*%Sig # model submatrix for the monotone term
-  object$X <- X # the finished model matrix
+  # get matrix Sigma and remove the first column for the monotone smooth (column and row for Sig)
+##  Sig <- matrix(as.numeric(rep(1:q,q)>=rep(1:q,each=q)),q,q) ## coef summation matrix
+  Sig <- matrix(1,q,q)  ## coef summation matrix
+  Sig[upper.tri(Sig)] <-0
+  X <- X1[,-1]%*%Sig[-1,-1] # drop intercept term, model submatrix for the monotone term
+  object$X <- X # the final model matrix
   object$P <- list()
   object$S <- list()
-  object$Sigma <- Sig
+  object$Sigma <- Sig[-1,-1]
 
   if (!object$fixed) # create the penalty matrix
   { P <- diff(diag(q-1),difference=1)
     object$P[[1]] <- P
     object$S[[1]] <- crossprod(P)
   }
-  b <- rep(1,q-1) # define vector of 0's & 1's for model parameters identification
-  object$p.ident <- b  
+  object$p.ident <- rep(TRUE,q-1) ## p.ident is an indicator of which coefficients must be positive (exponentiated)
   object$rank <- ncol(object$X)-1  # penalty rank
   object$null.space.dim <- m+1  # dim. of unpenalized space
   object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
@@ -168,9 +96,9 @@ Predict.matrix.mpi.smooth<-function(object,data)
 ## prediction method function for the `mpi' smooth class
 { m <- object$m+1; # spline order, m+1=3 default for cubic spline
   q <- object$df +1
-  Sig <- matrix(0,q,q)   # Define Matrix Sigma
-  # elements of matrix Sigma for increasing smooth
-  for (i in 1:q)  Sig[i,1:i] <- 1
+ # Sig <- matrix(as.numeric(rep(1:q,q)>=rep(1:q,each=q)),q,q) ## coef summation matrix
+  Sig <- matrix(1,q,q)  ## coef summation matrix
+  Sig[upper.tri(Sig)] <-0
   ## find spline basis inner knot range...
   ll <- object$knots[m+1];ul <- object$knots[length(object$knots)-m]
   m <- m + 1
@@ -195,12 +123,10 @@ Predict.matrix.mpi.smooth<-function(object,data)
   X
 }
 
-
 ########################################################
 ### Adding Monotone decreasing SCOP-spline construction 
 ########################################################
 
-
 smooth.construct.mpd.smooth.spec<- function(object, data, knots)
 ## construction of the monotone decreasing smooth
 { 
@@ -225,13 +151,15 @@ smooth.construct.mpd.smooth.spec<- function(object, data, knots)
   stop(paste("there should be ",nk," supplied knots"))
   #  get model matrix-------------
   X1 <- splineDesign(xk,x,ord=m+2)
-  # use matrix Sigma and remove the first column for the monotone smooth
-  Sig <- matrix(0,(q-1),(q-1))   # Define Matrix Sigma
-      # elements of matrix Sigma for decreasing smooth
-  for (i in 1:(q-1))  Sig[i,1:i]<- -1
-  X <- X1[,2:q]%*%Sig # model submatrix for the monotone term
+  # get matrix Sigma and remove the first column for the monotone smooth (column and row for Sig)
+ # Sig <- matrix(as.numeric(rep(1:q,q)>=rep(1:q,each=q)),q,q) ## coef summation matrix
+  Sig <- matrix(-1,q,q)  ## coef summation matrix
+  Sig[upper.tri(Sig)] <-0
+  Sig[,1] <- -Sig[,1] ## monotone decrease case
+
+  X <- X1[,-1]%*%Sig[-1,-1] # drop intercept term
   object$X<-X # the finished model matrix
-  object$Sigma <- Sig
+  object$Sigma <- Sig[-1,-1]
   object$P <- list()
   object$S <- list()
 
@@ -240,8 +168,7 @@ smooth.construct.mpd.smooth.spec<- function(object, data, knots)
     object$P[[1]] <- P
     object$S[[1]] <- crossprod(P)
   }
-  b<-rep(1,q-1) # define vector of 0's & 1's for model parameters identification
-  object$p.ident <- b  
+  object$p.ident <- rep(TRUE,q-1)  ## p.ident is an indicator of which coefficients must be positive (exponentiated) 
   object$rank <- ncol(object$X)-1  # penalty rank
   object$null.space.dim <- m+1  # dim. of unpenalized space
   object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
@@ -265,10 +192,11 @@ Predict.matrix.mpd.smooth<-function(object,data)
 ## prediction method function for the `mpd' smooth class
 { m <- object$m+1; # spline order, m+1=3 default for cubic spline
   q <- object$df +1
-  Sig <- matrix(0,q,q)   
   # elements of matrix Sigma for decreasing smooth...
-  Sig[,1] <- 1
-  for (i in 2:q)  Sig[i,2:i]<- -1 
+ # Sig <- matrix(as.numeric(rep(1:q,q)>=rep(1:q,each=q)),q,q) ## coef summation matrix
+  Sig <- matrix(-1,q,q)  ## coef summation matrix
+  Sig[upper.tri(Sig)] <-0
+  Sig[,1] <- -Sig[,1] ## monotone decrease case
   ## find spline basis inner knot range...
   ll <- object$knots[m+1];ul <- object$knots[length(object$knots)-m]
   m <- m + 1
@@ -345,8 +273,7 @@ smooth.construct.mdcv.smooth.spec<- function(object, data, knots)
     S[2:(q-1),2:(q-1)] <- crossprod(P)
     object$S[[1]] <- S
   }
-  b<-rep(1,q-1) # define vector of 0's & 1's for model parameters identification
-  object$p.ident <- b  
+  object$p.ident <- rep(TRUE,q-1)  ## p.ident is an indicator of which coefficients must be positive (exponentiated)  
   object$rank <- ncol(object$X)-1  # penalty rank
   object$null.space.dim <- m+1  # dim. of unpenalized space
   object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
@@ -449,8 +376,7 @@ smooth.construct.mdcx.smooth.spec<- function(object, data, knots)
     S[2:(q-1),2:(q-1)] <- crossprod(P)
     object$S[[1]] <- S
   }
-  b<-rep(1,q-1) # define vector of 0's & 1's for model parameters identification
-  object$p.ident <- b  
+  object$p.ident <- rep(TRUE,q-1)  ## p.ident is an indicator of which coefficients must be positive (exponentiated)  
   object$rank <- ncol(object$X)-1  # penalty rank
   object$null.space.dim <- m+1  # dim. of unpenalized space
   object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
@@ -562,8 +488,7 @@ smooth.construct.micv.smooth.spec<- function(object, data, knots)
     S[2:(q-1),2:(q-1)] <- crossprod(P)
     object$S[[1]] <- S
   }
-  b<-rep(1,q-1) # define vector of 0's & 1's for model parameters identification
-  object$p.ident <- b  
+  object$p.ident <- rep(TRUE,q-1)  ## p.ident is an indicator of which coefficients must be positive (exponentiated) 
   object$rank <- ncol(object$X)-1  # penalty rank
   object$null.space.dim <- m+1  # dim. of unpenalized space
   object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
@@ -669,8 +594,7 @@ smooth.construct.micx.smooth.spec<- function(object, data, knots)
     S[2:(q-1),2:(q-1)] <- crossprod(P)
     object$S[[1]] <- S
   }
-  b<-rep(1,q-1) # define vector of 0's & 1's for model parameters identification
-  object$p.ident <- b  
+  object$p.ident <- rep(TRUE,q-1) ## p.ident is an indicator of which coefficients must be positive (exponentiated) 
   object$rank <- ncol(object$X)-1  # penalty rank
   object$null.space.dim <- m+1  # dim. of unpenalized space
   object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
@@ -774,8 +698,7 @@ smooth.construct.cv.smooth.spec<- function(object, data, knots)
     S[2:(q-1),2:(q-1)] <- crossprod(P)
     object$S[[1]] <- S
   }
-  b<-rep(1,q-1) # define vector of 0's & 1's for model parameters identification
-  object$p.ident <- b  
+  object$p.ident <- rep(TRUE,q-1) ## p.ident is an indicator of which coefficients must be positive (exponentiated) 
   object$rank <- ncol(object$X)-1  # penalty rank
   object$null.space.dim <- m+1  # dim. of unpenalized space
   object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
@@ -875,8 +798,7 @@ smooth.construct.cx.smooth.spec<- function(object, data, knots)
     S[2:(q-1),2:(q-1)] <- crossprod(P)
     object$S[[1]] <- S
   }
-  b<-rep(1,q-1) # define vector of 0's & 1's for model parameters identification
-  object$p.ident <- b  
+  object$p.ident <- rep(TRUE,q-1) ## p.ident is an indicator of which coefficients must be positive (exponentiated) 
   object$rank <- ncol(object$X)-1  # penalty rank
   object$null.space.dim <- m+1  # dim. of unpenalized space
   object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
@@ -930,13 +852,14 @@ Predict.matrix.cx.smooth<-function(object,data)
 }
 
 
-####################################################
-### Adding a SCOP-spline construction with positivity constraint 
+
+#####################################################
+### Adding positive SCOP-spline construction 
 ######################################################
 
 
 smooth.construct.po.smooth.spec<- function(object, data, knots)
-## construction of the positive smooth...
+## construction of the positivelt costrained smooth
 { 
   m <- object$p.order[1]
   if (is.na(m)) m <- 2 ## default for cubic spline
@@ -958,49 +881,50 @@ smooth.construct.po.smooth.spec<- function(object, data, knots)
   stop(paste("there should be ", nk," supplied knots"))
   #  get model matrix-------------
   X1 <- splineDesign(xk,x,ord=m+2)
-  # use matrix Sigma which is just an identitymatrix for positive constraint 
-  Sig <- diag(1,q)   
-  
- # X <- X1[,2:q]%*%Sig # model submatrix for the positive term
-   X <- X1
+  # use matrix Sigma and remove the first column for the positive smooth
+  Sig <- diag(1,(q-1))  ## identity matrix 
+ ## Sig <- diag(1,q)  ## identity matrix
+     
+  X <- X1[,2:q] 
+ ## X <- X1
   object$X <- X # the finished model matrix
   object$P <- list()
   object$S <- list()
   object$Sigma <- Sig
 
   if (!object$fixed) # create the penalty matrix
-  { P <- diff(diag(q),difference=1)
+  { P <- diff(diag(q-1),difference=1)
+   ## P <- diff(diag(q),difference=1)
     object$P[[1]] <- P
     object$S[[1]] <- crossprod(P)
   }
-  b <- rep(1,q-1) # define vector of 0's & 1's for model parameters identification
-  object$p.ident <- b  
+  object$p.ident <- rep(TRUE,q-1) ## p.ident is an indicator of which coefficients must be positive (exponentiated)
+ ## object$p.ident <- rep(TRUE,q)
   object$rank <- ncol(object$X)-1  # penalty rank
+ ## object$rank <- ncol(object$X)  # penalty rank
   object$null.space.dim <- m+1  # dim. of unpenalized space
- # object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
+  object$C <- matrix(0, 0, ncol(X)) # to have no other constraints 
   object$knots <- xk;
   object$m <- m;
   object$df<-ncol(object$X)     # maximum DoF (if unconstrained)
 
   ## get model matrix for 1st and 2nd derivatives of the smooth...
   h <- (max(x)-min(x))/(q-m-1) ## distance between two adjacent knots
-  object$Xdf1 <- splineDesign(xk,x,ord=m+1)[,1:(q-1)]/h ## ord is by one less for the 1st derivative
-  object$Xdf2 <- splineDesign(xk,x,ord=m)[,1:(q-2)]/h^2 ## ord is by two less for the 2nd derivative
+  object$Xdf1 <- splineDesign(xk,x,ord=m+1)[,2:(q-1)]/h ## ord is by one less for the 1st derivative
+  object$Xdf2 <- splineDesign(xk,x,ord=m)[,2:(q-2)]/h^2 ## ord is by two less for the 2nd derivative
 
   class(object)<-"po.smooth"  # Give object a class
   object
 }
 
-
+## NOte: maybe no need to set the fisrt coefficient to zero, as not intercept is required in the model with 'po' smooth
 ## Prediction matrix for the `po` smooth class... 
 
-
 Predict.matrix.po.smooth<-function(object,data)
 ## prediction method function for the `po' smooth class
 { m <- object$m+1; # spline order, m+1=3 default for cubic spline
   q <- object$df +1
-  Sig <- diag(q)  # Define Matrix Sigma
- 
+ ## Sig <- diag(1,q)   # Define Matrix Sigma
   ## find spline basis inner knot range...
   ll <- object$knots[m+1];ul <- object$knots[length(object$knots)-m]
   m <- m + 1
@@ -1009,7 +933,6 @@ Predict.matrix.po.smooth<-function(object,data)
   ind <- x<=ul & x>=ll ## data in range
   if (sum(ind)==n) { ## all in range
      X <- spline.des(object$knots,x,m)$design
-   #  X <- X%*%Sig 
   } else { ## some extrapolation needed 
      ## matrix mapping coefs to value and slope at end points...
      D <- spline.des(object$knots,c(ll,ll,ul,ul),m,c(0,1,0,1))$design
@@ -1020,7 +943,6 @@ Predict.matrix.po.smooth<-function(object,data)
      if (sum(ind)>0) X[ind,] <- cbind(1,x[ind]-ll)%*%D[1:2,]
      ind <- x > ul
      if (sum(ind)>0) X[ind,] <- cbind(1,x[ind]-ul)%*%D[3:4,]
-   #  X <- X%*%Sig 
   }
   X
 }
@@ -1028,3 +950,5 @@ Predict.matrix.po.smooth<-function(object,data)
 
 
 
+
+
